import { Host, h } from "@stencil/core";
import { showToast } from "../../swc-toast/utils";
export class SwcFileUploader {
    /**
     * disabled prop for the component
     */
    disabled = false;
    /**
     * extended prop for the main file component
     */
    extended = true;
    /**
     * thumbnail prop for the main file component
     */
    thumbnail = 'file';
    /**
     * inverted prop for the file component
     */
    inverted = false;
    /**
     * hasError prop for the file component
     */
    hasError = false;
    /**
     * progress prop for the file component
     */
    progress = 0;
    /**
     * multiple prop for the file component
     */
    multiple = false;
    /**
     * type prop for the component
     */
    type = 'draggable';
    /**
     * maxFileSize prop for the component
     */
    maxFileSize = 0.5;
    /**
     * maxFileSize prop for the component
     */
    accepts = [];
    uploadStart;
    uploadRetry;
    uploadError;
    fileChange;
    //reactive states
    fileInputRef;
    files = [];
    selectedFileIndex = null;
    //computed properties for the getter
    get fileDetails() {
        return this.files.map((file) => ({
            name: file.name,
            size: (file.size / 1024 / 1024).toFixed(2),
        }));
    }
    get formattedAcceptedFileTypes() {
        if (!Array.isArray(this.accepts) || this.accepts.length === 0)
            return '';
        const formattedTypes = this.accepts.map((type) => type.replace('.', '').toUpperCase());
        const lastType = formattedTypes.pop();
        return this.accepts.length === 1
            ? `${lastType}.`
            : `${formattedTypes.join(', ')} or ${lastType}.`;
    }
    get maxFileSizeInfo() {
        return this.maxFileSize ? `Max size of ${this.maxFileSize} MB` : '';
    }
    //methods for the main component
    resetInputValueToNull = (event) => {
        ;
        event.target.value = '';
    };
    replaceMultipleFile = (event) => {
        const targetEl = event.target;
        const newFile = targetEl.files ? targetEl.files[0] : null;
        if (newFile && this.selectedFileIndex !== null) {
            this.files = this.files.map((unit, index) => {
                return this.selectedFileIndex == index ? newFile : unit;
            });
            //   this.files[this.selectedFileIndex] = newFile
            this.selectedFileIndex = null;
        }
        this.resetInputValueToNull(event);
        this.checkMaxFileSize();
    };
    onChange = (e) => {
        this.uploadStart.emit();
        const files = this.fileInputRef?.files;
        const selectedFiles = files ? Array.from(files) : [];
        if (this.multiple && this.selectedFileIndex !== null && selectedFiles.length > 0) {
            this.replaceMultipleFile(e);
        }
        if (!this.multiple && selectedFiles.length > 1) {
            this.uploadError.emit('Only one file can be uploaded when multiple is disabled.');
            showToast('error', 'Error', 'Only one file can be uploaded when multiple is disabled.');
        }
        if (this.multiple) {
            this.files = [...this.files, ...selectedFiles];
        }
        else {
            this.files = selectedFiles;
        }
        this.checkMaxFileSize();
    };
    triggerFileInput = (event) => {
        event.preventDefault();
        this.fileInputRef.click();
    };
    dragOver = (event) => {
        event.preventDefault();
        const target = event.currentTarget;
        if (!target.classList.contains('sfe-light-blue') && !this.disabled) {
            target.classList.add('sfe-light-blue');
        }
    };
    dragLeave = (event) => {
        event.preventDefault();
        const targetEl = event.currentTarget;
        targetEl.classList.remove('sfe-light-blue');
    };
    drop = (event) => {
        event.preventDefault();
        if (this.disabled || !event.dataTransfer) {
            return;
        }
        this.fileInputRef.files = event.dataTransfer.files;
        this.onChange(event);
        event.currentTarget.classList.remove('sfe-light-blue');
    };
    checkMaxFileSize = () => {
        if (this.maxFileSize) {
            const validFiles = new Set(); // Use Set to avoid duplicates
            const invalidFiles = [];
            this.files.forEach((file) => {
                const fileSizeToMb = file.size / 1024 / 1024;
                if (fileSizeToMb > this.maxFileSize) {
                    invalidFiles.push(file);
                }
                else {
                    validFiles.add(file); // Use Set to prevent duplicates
                }
            });
            if (invalidFiles.length > 0) {
                this.uploadError.emit(`Some files exceed the size limit of ${this.maxFileSize} MB.`);
                showToast('error', 'Error', `The following files exceed the size limit of ${this.maxFileSize} MB: ${invalidFiles
                    .map((f) => f.name)
                    .join(', ')}`);
            }
            this.files = Array.from(validFiles); // Convert Set back to Array
            this.fileChange.emit(this.files);
        }
    };
    removeFile = (event) => {
        const index = event.detail;
        if (index > -1 && index < this.files.length) {
            this.files = this.files.filter((_, idx) => idx !== index);
            this.fileChange.emit(this.files);
        }
    };
    handleFileReplace = (event) => {
        const targetEl = event.target;
        const index = event.detail;
        this.selectedFileIndex = index;
        targetEl.value = '';
        this.triggerFileInput(event);
    };
    render() {
        return (h(Host, { key: '5b8f1317817b09d73001e2efe96a706d3142217b' }, h("div", { key: 'd2a316bb63816c9aeeaedbcee5654997af0a40e5', class: "d-block", "data-testid": "sfe-file-uploader" }, h("input", { key: '7b793e013c5f750bf663dce482a57f91878e3802', ref: (el) => (this.fileInputRef = el), type: "file", multiple: this.multiple, disabled: this.disabled, accept: this.accepts.join(','), style: { display: 'none' }, onClick: this.resetInputValueToNull, onChange: this.onChange }), (!this.multiple ? this.fileDetails.length === 0 : true) && (h("div", { key: '1df94eed18eb2a892965e690f34e157e21803af8' }, this.type === 'basic' && (h("button", { key: 'e15ef8b5a4099a0feea35b374221e339bfe4dad9', "aria-label": "choose file", disabled: this.disabled, class: {
                'basic-uploader': true,
                'has-error': this.hasError,
            }, onClick: this.triggerFileInput }, h("span", { key: '2d80dec21da3caeaf288ffd5afba5e846437882a', class: "chooser d-flex align-items-center" }, "Choose file..."))), this.type !== 'basic' && (h("button", { key: '378c375d885e9e423e9b993fbec8d9bf20f0e17e', class: {
                'drag-area': true,
                disabled: this.disabled,
                'not-extended': !this.extended,
                'has-error': this.hasError,
            }, disabled: this.disabled, onDragOver: this.dragOver, onDragLeave: this.dragLeave, onDrop: this.drop, onClick: this.triggerFileInput }, h("div", { key: '213248e981801fcd4d1dac5da33cf49de720f1cc' }, h("div", { key: '4bce68ad6ed1a0539c45d72bfbd34c14f6888239', class: "drag-label" }, "Drag & Drop or ", h("span", { key: 'dcda6f6118705077c22ad8bcc8b0280f24e04fc7', class: "choose-file" }, "Choose file"), " to upload"), this.extended && (h("small", { key: '8b202ccb67880af0cb34ee1c961bfb2f0d04d66e', class: "formats" }, this.formattedAcceptedFileTypes, " ", this.maxFileSizeInfo))))))), !this.multiple && this.fileDetails.length > 0 && (h("swc-single-file", { key: '66802e4f304e69416141ef3535736f827420d16f', inverted: this.inverted, hasError: this.hasError, fileInfo: this.files, extended: this.extended, thumbnail: this.thumbnail, progress: this.progress, onUpdateRemove: this.removeFile, onUpdateReplace: this.triggerFileInput, onUploadRetry: () => this.uploadRetry.emit() })), this.multiple && this.fileDetails.length > 0 && (h("swc-multiple-file", { key: '09f2edbf03545065e94d4a62be57c704498bb811', inverted: this.inverted, hasError: this.hasError, fileInfo: this.files, progress: this.progress, onUpdateRemove: this.removeFile, onUpdateReplace: this.handleFileReplace, onUploadRetry: () => this.uploadRetry.emit() })))));
    }
    static get is() { return "swc-file-uploader"; }
    static get encapsulation() { return "scoped"; }
    static get originalStyleUrls() {
        return {
            "$": ["swc-file-uploader.scss"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["swc-file-uploader.css"]
        };
    }
    static get properties() {
        return {
            "disabled": {
                "type": "boolean",
                "attribute": "disabled",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "disabled prop for the component"
                },
                "getter": false,
                "setter": false,
                "reflect": true,
                "defaultValue": "false"
            },
            "extended": {
                "type": "boolean",
                "attribute": "extended",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "extended prop for the main file component"
                },
                "getter": false,
                "setter": false,
                "reflect": true,
                "defaultValue": "true"
            },
            "thumbnail": {
                "type": "string",
                "attribute": "thumbnail",
                "mutable": false,
                "complexType": {
                    "original": "'none' | 'file' | 'photo'",
                    "resolved": "\"file\" | \"none\" | \"photo\"",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "thumbnail prop for the main file component"
                },
                "getter": false,
                "setter": false,
                "reflect": true,
                "defaultValue": "'file'"
            },
            "inverted": {
                "type": "boolean",
                "attribute": "inverted",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "inverted prop for the file component"
                },
                "getter": false,
                "setter": false,
                "reflect": true,
                "defaultValue": "false"
            },
            "hasError": {
                "type": "boolean",
                "attribute": "has-error",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "hasError prop for the file component"
                },
                "getter": false,
                "setter": false,
                "reflect": true,
                "defaultValue": "false"
            },
            "progress": {
                "type": "number",
                "attribute": "progress",
                "mutable": false,
                "complexType": {
                    "original": "number | number",
                    "resolved": "number",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "progress prop for the file component"
                },
                "getter": false,
                "setter": false,
                "reflect": true,
                "defaultValue": "0"
            },
            "multiple": {
                "type": "boolean",
                "attribute": "multiple",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "multiple prop for the file component"
                },
                "getter": false,
                "setter": false,
                "reflect": true,
                "defaultValue": "false"
            },
            "type": {
                "type": "string",
                "attribute": "type",
                "mutable": false,
                "complexType": {
                    "original": "'draggable' | 'basic'",
                    "resolved": "\"basic\" | \"draggable\"",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "type prop for the component"
                },
                "getter": false,
                "setter": false,
                "reflect": true,
                "defaultValue": "'draggable'"
            },
            "maxFileSize": {
                "type": "number",
                "attribute": "max-file-size",
                "mutable": false,
                "complexType": {
                    "original": "number",
                    "resolved": "number",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "maxFileSize prop for the component"
                },
                "getter": false,
                "setter": false,
                "reflect": false,
                "defaultValue": "0.5"
            },
            "accepts": {
                "type": "unknown",
                "attribute": "accepts",
                "mutable": false,
                "complexType": {
                    "original": "any[]",
                    "resolved": "any[]",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "maxFileSize prop for the component"
                },
                "getter": false,
                "setter": false,
                "defaultValue": "[]"
            }
        };
    }
    static get states() {
        return {
            "files": {},
            "selectedFileIndex": {}
        };
    }
    static get events() {
        return [{
                "method": "uploadStart",
                "name": "uploadStart",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "complexType": {
                    "original": "null",
                    "resolved": "null",
                    "references": {}
                }
            }, {
                "method": "uploadRetry",
                "name": "uploadRetry",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "complexType": {
                    "original": "null",
                    "resolved": "null",
                    "references": {}
                }
            }, {
                "method": "uploadError",
                "name": "uploadError",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                }
            }, {
                "method": "fileChange",
                "name": "fileChange",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "complexType": {
                    "original": "File[]",
                    "resolved": "File[]",
                    "references": {
                        "File": {
                            "location": "global",
                            "id": "global::File"
                        }
                    }
                }
            }];
    }
}
//# sourceMappingURL=swc-file-uploader.js.map
