import { h } from "@stencil/core";
import tippy from "tippy.js";
/**
 const emit = defineEmits(['view:previous', 'view:next', 'dismiss:tour', 'end:tour'])
 */
export class SwcTourGuide {
    hostElement;
    /**
     * determines whether the tour should begin
     */
    startTour = false;
    /**
     * determines whether the tour can be dismissed
     */
    dismissible = true;
    /**
     * determines whether an overlay should be displayed
     */
    useModalOverlay = true;
    /**
     * determines whether a tour step should be highlighted
     */
    useHighlightSteps = true;
    /**
     * determines whether an arrow should be displayed
     */
    showArrow = true;
    /**
     * variant of the tour guide
     */
    variant = 'light';
    /**
     * steps for the tour guide
     */
    steps = [];
    /**
     * emitted when the previous button is clicked
     */
    viewPrevious;
    /**
     * emitted when the next button is clicked
     */
    viewNext;
    /**
     * emitted when the next button is clicked
     */
    dismissTour;
    /**
     * emitted when the next button is clicked
     */
    endTour;
    instance; // Instance<Props>[] & Instance<Props>
    currentIndex = 0;
    activeNode;
    contentRef;
    get currentStep() {
        return this.steps[this.currentIndex];
    }
    get hasPrevious() {
        return this.steps.length > 1 && this.currentIndex > 0;
    }
    get hasNext() {
        return this.steps.length > 1 && this.currentIndex < this.steps.length - 1;
    }
    viewPreviousStep = () => {
        this.currentIndex -= 1;
        this.viewPrevious.emit();
        this.removeHighlight();
        this.reInitialize();
    };
    viewNextStep = () => {
        this.currentIndex += 1;
        this.viewNext.emit();
        this.removeHighlight();
        this.reInitialize();
    };
    handleDismissTour = () => {
        this.dismissTour.emit();
    };
    handleEndTour = () => {
        this.endTour.emit();
    };
    hideTour = () => {
        this.instance[0].hide();
        this.instance[0].unmount();
        this.currentIndex = 0;
        this.removeHighlight();
    };
    getAllParentElements = () => {
        if (!this.activeNode)
            return [];
        let element = this.activeNode;
        const parents = [];
        while (element.parentNode) {
            element = element.parentNode;
            if (element.nodeType === 1) {
                parents.push(element);
            }
        }
        return parents;
    };
    reInitialize = () => {
        this.instance[0].hide();
        // create a new instance using the details of the next step.
        this.init();
        this.instance[0].show();
        this.activeNode = document.querySelector(`${this.currentStep.cssSelector}`);
        if (this.activeNode) {
            this.activeNode.scrollIntoView({
                behavior: 'smooth',
                block: 'center',
            });
        }
        if (this.useHighlightSteps) {
            this.highlightStep();
        }
    };
    highlightStep = () => {
        if (!this.activeNode)
            return;
        if (!this.activeNode.style.position) {
            this.activeNode.style.position = 'relative';
        }
        if (!this.activeNode.style.backgroundColor) {
            this.activeNode.classList.add('sfe-active-tour-step-bg');
        }
        this.activeNode.classList.add('sfe-active-tour-step');
        /**
         * Ensures that the highlighted element stays on top of the overlay.
         * Hence temporarily disables z-index on all of its parent elements.
         * This ensures the highlighted element maintains its own stacking context.
         */
        const parentElements = this.getAllParentElements();
        if (parentElements && parentElements.length > 0) {
            for (const element of parentElements) {
                element.classList.add('sfe-no-z-index');
            }
        }
    };
    removeHighlight = () => {
        if (this.activeNode && this.useHighlightSteps) {
            this.activeNode.classList.remove('sfe-active-tour-step');
            this.activeNode.classList.remove('sfe-active-tour-step-bg');
            const parentElements = this.getAllParentElements();
            if (parentElements && parentElements.length > 0) {
                for (const element of parentElements) {
                    element.classList.remove('sfe-no-z-index');
                }
            }
        }
    };
    startTourHandler(newValue) {
        if (newValue) {
            this.reInitialize();
        }
        else {
            this.hideTour();
        }
    }
    init = () => {
        this.instance = tippy(this.hostElement.querySelector(`${this.currentStep.cssSelector}`), {
            theme: this.variant,
            content: this.contentRef,
            animation: 'shift-toward',
            placement: this.currentStep.position || 'bottom',
            allowHTML: true,
            trigger: 'manual',
            hideOnClick: false,
            arrow: this.showArrow,
            interactive: true,
            role: 'dialog',
            maxWidth: 'none',
            // helps to hide a warning on the console
            appendTo: document.body,
            // sets exit animation duration to zero to ensure instant exit
            duration: [300, 0],
            zIndex: 100099,
        });
    };
    componentDidLoad() {
        this.init();
    }
    disconnectedCallback() {
        this.instance[0].unmount();
    }
    render() {
        return (h("div", { key: 'b44e348dee79224bf008296c64baf61bba1d3fc0', class: "tour-guide-item-backdrop", "data-show": this.useModalOverlay && this.startTour ? 'true' : 'false', "data-testid": "sfe-tour-guide" }, h("div", { key: '0abdfa6d9b69da6c632aa73bd123d17fb6715d90', ref: (el) => (this.contentRef = el), "aria-live": "polite" }, h("swc-tour-guide-step", { key: 'a22f4211844097ff513c4b92782a80e3e9bbc051', heading: this.currentStep.title, content: this.currentStep.content, hasPrevious: this.hasPrevious, hasNext: this.hasNext, isLastStep: this.currentIndex === this.steps.length - 1, currentIndex: this.currentIndex, numberOfSteps: this.steps.length, variant: this.variant, startTour: this.startTour, dismissible: this.dismissible, onViewPrevious: this.viewPreviousStep, onViewNext: this.viewNextStep, onDismissTour: this.handleDismissTour, onEndTour: this.handleEndTour }))));
    }
    static get is() { return "swc-tour-guide"; }
    static get originalStyleUrls() {
        return {
            "$": ["swc-tour-guide.scss"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["swc-tour-guide.css"]
        };
    }
    static get properties() {
        return {
            "startTour": {
                "type": "boolean",
                "attribute": "start-tour",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "determines whether the tour should begin"
                },
                "getter": false,
                "setter": false,
                "reflect": false,
                "defaultValue": "false"
            },
            "dismissible": {
                "type": "boolean",
                "attribute": "dismissible",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "determines whether the tour can be dismissed"
                },
                "getter": false,
                "setter": false,
                "reflect": false,
                "defaultValue": "true"
            },
            "useModalOverlay": {
                "type": "boolean",
                "attribute": "use-modal-overlay",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "determines whether an overlay should be displayed"
                },
                "getter": false,
                "setter": false,
                "reflect": false,
                "defaultValue": "true"
            },
            "useHighlightSteps": {
                "type": "boolean",
                "attribute": "use-highlight-steps",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "determines whether a tour step should be highlighted"
                },
                "getter": false,
                "setter": false,
                "reflect": false,
                "defaultValue": "true"
            },
            "showArrow": {
                "type": "boolean",
                "attribute": "show-arrow",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "determines whether an arrow should be displayed"
                },
                "getter": false,
                "setter": false,
                "reflect": false,
                "defaultValue": "true"
            },
            "variant": {
                "type": "string",
                "attribute": "variant",
                "mutable": false,
                "complexType": {
                    "original": "'light' | 'brand'",
                    "resolved": "\"brand\" | \"light\"",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "variant of the tour guide"
                },
                "getter": false,
                "setter": false,
                "reflect": false,
                "defaultValue": "'light'"
            },
            "steps": {
                "type": "unknown",
                "attribute": "steps",
                "mutable": false,
                "complexType": {
                    "original": "{ cssSelector: string; title: string; content: string; position?: Placement }[]",
                    "resolved": "{ cssSelector: string; title: string; content: string; position?: Placement | undefined; }[]",
                    "references": {
                        "Placement": {
                            "location": "import",
                            "path": "tippy.js",
                            "id": "../../node_modules/tippy.js/index.d.ts::Placement"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "steps for the tour guide"
                },
                "getter": false,
                "setter": false,
                "defaultValue": "[]"
            }
        };
    }
    static get states() {
        return {
            "instance": {},
            "currentIndex": {}
        };
    }
    static get events() {
        return [{
                "method": "viewPrevious",
                "name": "viewPrevious",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": "emitted when the previous button is clicked"
                },
                "complexType": {
                    "original": "any",
                    "resolved": "any",
                    "references": {}
                }
            }, {
                "method": "viewNext",
                "name": "viewNext",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": "emitted when the next button is clicked"
                },
                "complexType": {
                    "original": "any",
                    "resolved": "any",
                    "references": {}
                }
            }, {
                "method": "dismissTour",
                "name": "dismissTour",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": "emitted when the next button is clicked"
                },
                "complexType": {
                    "original": "any",
                    "resolved": "any",
                    "references": {}
                }
            }, {
                "method": "endTour",
                "name": "endTour",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": "emitted when the next button is clicked"
                },
                "complexType": {
                    "original": "any",
                    "resolved": "any",
                    "references": {}
                }
            }];
    }
    static get elementRef() { return "hostElement"; }
    static get watchers() {
        return [{
                "propName": "startTour",
                "methodName": "startTourHandler"
            }];
    }
}
//# sourceMappingURL=swc-tour-guide.js.map
